import sys
from enum import EnumMeta
from importlib import import_module
from pathlib import Path
from textwrap import dedent
from typing import TYPE_CHECKING

from jinja2 import pass_context
from qtpy.QtCore import QObject, Signal

if TYPE_CHECKING:
    from mkdocs_macros.plugin import MacrosPlugin

EXAMPLES = Path(__file__).parent.parent / "examples"
IMAGES = Path(__file__).parent / "_auto_images"
IMAGES.mkdir(exist_ok=True, parents=True)


def define_env(env: "MacrosPlugin"):
    @env.macro
    @pass_context
    def show_widget(context, width: int = 500) -> list[Path]:
        # extract all fenced code blocks starting with "python"
        page = context["page"]
        dest = IMAGES / f"{page.title}.png"
        if "build" in sys.argv:
            dest.unlink(missing_ok=True)

            codeblocks = [
                b[6:].strip()
                for b in page.markdown.split("```")
                if b.startswith("python")
            ]
            src = codeblocks[0].strip()
            src = src.replace(
                "QApplication([])", "QApplication.instance() or QApplication([])"
            )
            src = src.replace("app.exec_()", "")

            exec(src)
            _grab(dest, width)
        return (
            f"![{page.title}](../{dest.parent.name}/{dest.name})"
            f"{{ loading=lazy; width={width} }}\n\n"
        )

    @env.macro
    def show_members(cls: str):
        # import class
        module, name = cls.rsplit(".", 1)
        _cls = getattr(import_module(module), name)

        first_q = next(
            (
                b.__name__
                for b in _cls.__mro__
                if issubclass(b, QObject) and ".Qt" in b.__module__
            ),
            None,
        )

        inherited_members = set()
        for base in _cls.__mro__:
            if issubclass(base, QObject) and ".Qt" in base.__module__:
                inherited_members.update(
                    {k for k in dir(base) if not k.startswith("_")}
                )

        new_signals = {
            k
            for k, v in vars(_cls).items()
            if not k.startswith("_") and isinstance(v, Signal)
        }

        self_members = {
            k
            for k in dir(_cls)
            if not k.startswith("_") and k not in inherited_members | new_signals
        }

        enums = []
        for m in list(self_members):
            if isinstance(getattr(_cls, m), EnumMeta):
                self_members.remove(m)
                enums.append(m)

        out = ""
        if first_q:
            url = f"https://doc.qt.io/qt-6/{first_q.lower()}.html"
            out += f"## Qt Class\n\n<a href='{url}'>`{first_q}`</a>\n\n"

        out += ""

        if new_signals:
            out += "## Signals\n\n"
            for sig in new_signals:
                out += f"### `{sig}`\n\n"

        if enums:
            out += "## Enums\n\n"
            for e in enums:
                out += f"### `{_cls.__name__}.{e}`\n\n"
                for m in getattr(_cls, e):
                    out += f"- `{m.name}`\n\n"

        if self_members:

            out += dedent(
                f"""
            ## Methods

            ::: {cls}
                options:
                  heading_level: 3
                  show_source: False
                  show_inherited_members: false
                  show_signature_annotations: True
                  members: {sorted(self_members)}
                  docstring_style: numpy
                  show_bases: False
                  show_root_toc_entry: False
                  show_root_heading: False
            """
            )

        return out


def _grab(dest: str | Path, width) -> list[Path]:
    """Grab the top widgets of the application."""
    from qtpy.QtCore import QTimer
    from qtpy.QtWidgets import QApplication

    w = QApplication.topLevelWidgets()[-1]
    w.setFixedWidth(width)
    w.activateWindow()
    w.setMinimumHeight(40)
    w.grab().save(str(dest))

    # hack to make sure the object is truly closed and deleted
    while True:
        QTimer.singleShot(10, w.deleteLater)
        QApplication.processEvents()
        try:
            w.parent()
        except RuntimeError:
            return



import inspect
import os
import time
from concurrent.futures import Future, TimeoutError

import pytest
from qtpy.QtCore import QCoreApplication, QObject, QThread, Signal

from superqt.utils import ensure_main_thread, ensure_object_thread

skip_on_ci = pytest.mark.skipif(bool(os.getenv("CI")), reason="github hangs")


class SampleObject(QObject):
    assigment_done = Signal()

    def __init__(self):
        super().__init__()
        self.main_thread_res = {}
        self.object_thread_res = {}
        self.main_thread_prop_val = None
        self.sample_thread_prop_val = None

    def long_wait(self):
        time.sleep(1)

    @property
    def sample_main_thread_property(self):
        return self.main_thread_prop_val

    @sample_main_thread_property.setter  # type: ignore
    @ensure_main_thread()
    def sample_main_thread_property(self, value):
        if QThread.currentThread() is not QCoreApplication.instance().thread():
            raise RuntimeError("Wrong thread")
        self.main_thread_prop_val = value
        self.assigment_done.emit()

    @property
    def sample_object_thread_property(self):
        return self.sample_thread_prop_val

    @sample_object_thread_property.setter  # type: ignore
    @ensure_object_thread()
    def sample_object_thread_property(self, value):
        if QThread.currentThread() is not self.thread():
            raise RuntimeError("Wrong thread")
        self.sample_thread_prop_val = value
        self.assigment_done.emit()

    @ensure_main_thread
    def check_main_thread(self, a, *, b=1):
        if QThread.currentThread() is not QCoreApplication.instance().thread():
            raise RuntimeError("Wrong thread")
        self.main_thread_res = {"a": a, "b": b}
        self.assigment_done.emit()

    @ensure_object_thread
    def check_object_thread(self, a, *, b=1):
        if QThread.currentThread() is not self.thread():
            raise RuntimeError("Wrong thread")
        self.object_thread_res = {"a": a, "b": b}
        self.assigment_done.emit()

    @ensure_object_thread(await_return=True)
    def check_object_thread_return(self, a):
        if QThread.currentThread() is not self.thread():
            raise RuntimeError("Wrong thread")
        return a * 7

    @ensure_object_thread(await_return=True, timeout=200)
    def check_object_thread_return_timeout(self, a):
        if QThread.currentThread() is not self.thread():
            raise RuntimeError("Wrong thread")
        time.sleep(1)
        return a * 7

    @ensure_object_thread(await_return=False)
    def check_object_thread_return_future(self, a: int):
        """sample docstring"""
        if QThread.currentThread() is not self.thread():
            raise RuntimeError("Wrong thread")
        time.sleep(0.4)
        return a * 7

    @ensure_main_thread(await_return=True)
    def check_main_thread_return(self, a):
        if QThread.currentThread() is not QCoreApplication.instance().thread():
            raise RuntimeError("Wrong thread")
        return a * 8


class LocalThread(QThread):
    def __init__(self, ob):
        super().__init__()
        self.ob = ob

    def run(self):
        assert QThread.currentThread() is not QCoreApplication.instance().thread()
        self.ob.check_main_thread(5, b=8)
        self.ob.main_thread_prop_val = "text2"


class LocalThread2(QThread):
    def __init__(self, ob):
        super().__init__()
        self.ob = ob
        self.executed = False

    def run(self):
        assert QThread.currentThread() is not QCoreApplication.instance().thread()
        assert self.ob.check_main_thread_return(5) == 40
        self.executed = True


def test_only_main_thread(qapp):
    ob = SampleObject()
    ob.check_main_thread(1, b=3)
    assert ob.main_thread_res == {"a": 1, "b": 3}
    ob.check_object_thread(2, b=4)
    assert ob.object_thread_res == {"a": 2, "b": 4}
    ob.sample_main_thread_property = 5
    assert ob.sample_main_thread_property == 5
    ob.sample_object_thread_property = 7
    assert ob.sample_object_thread_property == 7


def test_main_thread(qtbot):
    ob = SampleObject()
    t = LocalThread(ob)
    with qtbot.waitSignal(t.finished):
        t.start()

    assert ob.main_thread_res == {"a": 5, "b": 8}
    assert ob.sample_main_thread_property == "text2"


def test_main_thread_return(qtbot):
    ob = SampleObject()
    t = LocalThread2(ob)
    with qtbot.wait_signal(t.finished):
        t.start()
    assert t.executed


def test_names(qapp):
    ob = SampleObject()
    assert ob.check_object_thread.__name__ == "check_object_thread"
    assert ob.check_object_thread_return.__name__ == "check_object_thread_return"
    assert (
        ob.check_object_thread_return_timeout.__name__
        == "check_object_thread_return_timeout"
    )
    assert (
        ob.check_object_thread_return_future.__name__
        == "check_object_thread_return_future"
    )
    assert ob.check_object_thread_return_future.__doc__ == "sample docstring"
    signature = inspect.signature(ob.check_object_thread_return_future)
    assert len(signature.parameters) == 1
    assert list(signature.parameters.values())[0].name == "a"
    assert list(signature.parameters.values())[0].annotation == int
    assert ob.check_main_thread_return.__name__ == "check_main_thread_return"


@skip_on_ci
def test_object_thread_return(qtbot):
    ob = SampleObject()
    thread = QThread()
    thread.start()
    ob.moveToThread(thread)
    assert ob.check_object_thread_return(2) == 14
    assert ob.thread() is thread
    with qtbot.waitSignal(thread.finished):
        thread.quit()


@skip_on_ci
def test_object_thread_return_timeout(qtbot):
    ob = SampleObject()
    thread = QThread()
    thread.start()
    ob.moveToThread(thread)
    with pytest.raises(TimeoutError):
        ob.check_object_thread_return_timeout(2)
    with qtbot.waitSignal(thread.finished):
        thread.quit()


@skip_on_ci
def test_object_thread_return_future(qtbot):
    ob = SampleObject()
    thread = QThread()
    thread.start()
    ob.moveToThread(thread)
    future = ob.check_object_thread_return_future(2)
    assert isinstance(future, Future)
    assert future.result() == 14
    with qtbot.waitSignal(thread.finished):
        thread.quit()


@skip_on_ci
def test_object_thread(qtbot):
    ob = SampleObject()
    thread = QThread()
    thread.start()
    ob.moveToThread(thread)
    with qtbot.waitSignal(ob.assigment_done):
        ob.check_object_thread(2, b=4)
    assert ob.object_thread_res == {"a": 2, "b": 4}

    with qtbot.waitSignal(ob.assigment_done):
        ob.sample_object_thread_property = "text"

    assert ob.sample_object_thread_property == "text"
    assert ob.thread() is thread
    with qtbot.waitSignal(thread.finished):
        thread.quit()



try:
    from fonticon_fa5 import FA5S
except ImportError as e:
    raise type(e)(
        "This example requires the fontawesome fontpack:\n\n"
        "pip install git+https://github.com/tlambert03/fonticon-fontawesome5.git"
    )

from qtpy.QtCore import QSize
from qtpy.QtWidgets import QApplication, QPushButton

from superqt.fonticon import IconOpts, icon, pulse, spin

app = QApplication([])

btn = QPushButton()
btn.setIcon(
    icon(
        FA5S.smile,
        color="blue",
        states={
            "active": IconOpts(
                glyph_key=FA5S.spinner,
                color="red",
                scale_factor=0.5,
                animation=pulse(btn),
            ),
            "disabled": {"color": "green", "scale_factor": 0.8, "animation": spin(btn)},
        },
    )
)
btn.setIconSize(QSize(256, 256))
btn.show()


@btn.clicked.connect
def toggle_state():
    btn.setChecked(not btn.isChecked())


app.exec()



"""Adapted for python from the KDToolBox

https://github.com/KDAB/KDToolBox/tree/master/qt/KDSignalThrottler

MIT License

Copyright (C) 2019-2022 Klarälvdalens Datakonsult AB, a KDAB Group company,
info@kdab.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

from typing import Deque

from qtpy.QtCore import QRect, QSize, Qt, QTimer, Signal
from qtpy.QtGui import QPainter, QPen
from qtpy.QtWidgets import (
    QApplication,
    QCheckBox,
    QComboBox,
    QFormLayout,
    QGroupBox,
    QHBoxLayout,
    QLabel,
    QPushButton,
    QSizePolicy,
    QSpinBox,
    QVBoxLayout,
    QWidget,
)

from superqt.utils._throttler import (
    GenericSignalThrottler,
    QSignalDebouncer,
    QSignalThrottler,
)


class DrawSignalsWidget(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        self.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding)
        self.setAttribute(Qt.WA_OpaquePaintEvent)

        self._scrollTimer = QTimer(self)
        self._scrollTimer.setInterval(10)
        self._scrollTimer.timeout.connect(self._scroll)
        self._scrollTimer.start()

        self._signalActivations: Deque[int] = Deque()
        self._throttledSignalActivations: Deque[int] = Deque()

    def sizeHint(self):
        return QSize(400, 200)

    def addSignalActivation(self):
        self._signalActivations.appendleft(0)

    def addThrottledSignalActivation(self):
        self._throttledSignalActivations.appendleft(0)

    def _scroll(self):
        cutoff = self.width()
        self.scrollAndCut(self._signalActivations, cutoff)
        self.scrollAndCut(self._throttledSignalActivations, cutoff)

        self.update()

    def scrollAndCut(self, v: Deque[int], cutoff: int):
        x = 0
        L = len(v)
        for p in range(L):
            v[p] += 1
            if v[p] > cutoff:
                x = p
                break

        # TODO: fix this... delete old ones

    def paintEvent(self, event):
        p = QPainter(self)
        p.fillRect(self.rect(), Qt.white)

        h = self.height()
        h2 = h // 2
        w = self.width()

        self._drawSignals(p, self._signalActivations, Qt.red, 0, h2)
        self._drawSignals(p, self._throttledSignalActivations, Qt.blue, h2, h)

        p.drawText(
            QRect(0, 0, w, h2),
            Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter,
            "Source signal",
        )
        p.drawText(
            QRect(0, h2, w, h2),
            Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter,
            "Throttled signal",
        )

        p.save()
        pen = QPen()
        pen.setWidthF(2.0)
        p.drawLine(0, h2, w, h2)
        p.restore()

    def _drawSignals(self, p: QPainter, v: Deque[int], color, yStart, yEnd):
        p.save()
        pen = QPen()
        pen.setWidthF(2.0)
        pen.setColor(color)
        p.setPen(pen)

        for i in v:
            p.drawLine(i, yStart, i, yEnd)
        p.restore()


class DemoWidget(QWidget):
    signalToBeThrottled = Signal()
    _throttler: GenericSignalThrottler

    def __init__(self, parent=None) -> None:
        super().__init__(parent)
        self._createUi()
        self._throttler = None

        self._throttlerKindComboBox.currentIndexChanged.connect(self._createThrottler)
        self._createThrottler()

        self._throttlerTimeoutSpinBox.valueChanged.connect(self.setThrottlerTimeout)
        self.setThrottlerTimeout()

        self._mainButton.clicked.connect(self.signalToBeThrottled)

        self._autoTriggerTimer = QTimer(self)
        self._autoTriggerTimer.setTimerType(Qt.TimerType.PreciseTimer)
        self._autoTriggerCheckBox.clicked.connect(self._startOrStopAutoTriggerTimer)
        self._startOrStopAutoTriggerTimer()

        self._autoTriggerIntervalSpinBox.valueChanged.connect(
            self._setAutoTriggerTimeout
        )
        self._setAutoTriggerTimeout()

        self._autoTriggerTimer.timeout.connect(self.signalToBeThrottled)
        self.signalToBeThrottled.connect(self._drawSignalsWidget.addSignalActivation)

    def _createThrottler(self) -> None:
        if self._throttler is not None:
            self._throttler.deleteLater()
            del self._throttler

        if self._throttlerKindComboBox.currentIndex() < 2:
            cls = QSignalThrottler
        else:
            cls = QSignalDebouncer
        if self._throttlerKindComboBox.currentIndex() % 2:
            policy = QSignalThrottler.EmissionPolicy.Leading
        else:
            policy = QSignalThrottler.EmissionPolicy.Trailing

        self._throttler: GenericSignalThrottler = cls(policy, self)

        self._throttler.setTimerType(Qt.TimerType.PreciseTimer)
        self.signalToBeThrottled.connect(self._throttler.throttle)
        self._throttler.triggered.connect(
            self._drawSignalsWidget.addThrottledSignalActivation
        )

        self.setThrottlerTimeout()

    def setThrottlerTimeout(self):
        self._throttler.setTimeout(self._throttlerTimeoutSpinBox.value())

    def _startOrStopAutoTriggerTimer(self):
        shouldStart = self._autoTriggerCheckBox.isChecked()
        if shouldStart:
            self._autoTriggerTimer.start()
        else:
            self._autoTriggerTimer.stop()

        self._autoTriggerIntervalSpinBox.setEnabled(shouldStart)
        self._autoTriggerLabel.setEnabled(shouldStart)

    def _setAutoTriggerTimeout(self):
        timeout = self._autoTriggerIntervalSpinBox.value()
        self._autoTriggerTimer.setInterval(timeout)

    def _createUi(self):
        helpLabel = QLabel(self)
        helpLabel.setWordWrap(True)
        helpLabel.setText(
            "<h2>SignalThrottler example</h2>"
            "<p>This example demonstrates the differences between "
            "the different kinds of signal throttlers and debouncers."
        )

        throttlerKindGroupBox = QGroupBox("Throttler configuration", self)

        self._throttlerKindComboBox = QComboBox(throttlerKindGroupBox)
        self._throttlerKindComboBox.addItems(
            (
                "Throttler, trailing",
                "Throttler, leading",
                "Debouncer, trailing",
                "Debouncer, leading",
            )
        )

        self._throttlerTimeoutSpinBox = QSpinBox(throttlerKindGroupBox)
        self._throttlerTimeoutSpinBox.setRange(1, 5000)
        self._throttlerTimeoutSpinBox.setValue(500)
        self._throttlerTimeoutSpinBox.setSuffix(" ms")

        layout = QFormLayout(throttlerKindGroupBox)
        layout.addRow("Kind of throttler:", self._throttlerKindComboBox)
        layout.addRow("Timeout:", self._throttlerTimeoutSpinBox)
        throttlerKindGroupBox.setLayout(layout)

        buttonGroupBox = QGroupBox("Throttler activation")
        self._mainButton = QPushButton(("Press me!"), buttonGroupBox)

        self._autoTriggerCheckBox = QCheckBox("Trigger automatically")

        autoTriggerLayout = QHBoxLayout()
        self._autoTriggerLabel = QLabel("Interval", buttonGroupBox)
        self._autoTriggerIntervalSpinBox = QSpinBox(buttonGroupBox)
        self._autoTriggerIntervalSpinBox.setRange(1, 5000)
        self._autoTriggerIntervalSpinBox.setValue(100)
        self._autoTriggerIntervalSpinBox.setSuffix(" ms")

        autoTriggerLayout.setContentsMargins(0, 0, 0, 0)
        autoTriggerLayout.addWidget(self._autoTriggerLabel)
        autoTriggerLayout.addWidget(self._autoTriggerIntervalSpinBox)

        layout = QVBoxLayout(buttonGroupBox)
        layout.addWidget(self._mainButton)
        layout.addWidget(self._autoTriggerCheckBox)
        layout.addLayout(autoTriggerLayout)
        buttonGroupBox.setLayout(layout)

        resultGroupBox = QGroupBox("Result")
        self._drawSignalsWidget = DrawSignalsWidget(resultGroupBox)
        layout = QVBoxLayout(resultGroupBox)
        layout.addWidget(self._drawSignalsWidget)
        resultGroupBox.setLayout(layout)

        layout = QVBoxLayout(self)
        layout.addWidget(helpLabel)
        layout.addWidget(throttlerKindGroupBox)
        layout.addWidget(buttonGroupBox)
        layout.addWidget(resultGroupBox)

        self.setLayout(layout)


if __name__ == "__main__":
    app = QApplication([__name__])
    w = DemoWidget()
    w.resize(600, 600)
    w.show()
    app.exec_()



from unittest.mock import Mock

from qtpy.QtCore import QObject, Signal

from superqt.utils import signals_blocked


def test_signal_blocker(qtbot):
    """make sure context manager signal blocker works"""

    class Emitter(QObject):
        sig = Signal()

    obj = Emitter()
    receiver = Mock()
    obj.sig.connect(receiver)

    # make sure signal works
    with qtbot.waitSignal(obj.sig):
        obj.sig.emit()

    receiver.assert_called_once()
    receiver.reset_mock()

    with signals_blocked(obj):
        obj.sig.emit()
        qtbot.wait(10)

    receiver.assert_not_called()



from qtpy.QtWidgets import QApplication

from superqt import QSearchableListWidget

app = QApplication([])

slider = QSearchableListWidget()
slider.addItems(["foo", "bar", "baz", "foobar", "foobaz", "barbaz"])
slider.show()

app.exec_()



from qtpy.QtCore import Qt
from qtpy.QtWidgets import QApplication

from superqt import QDoubleSlider

app = QApplication([])

sld = QDoubleSlider(Qt.Orientation.Horizontal)
sld.setRange(0, 1)
sld.setValue(0.5)
sld.show()

app.exec_()



"""Example for QCollapsible"""
from qtpy.QtWidgets import QApplication, QLabel, QPushButton

from superqt import QCollapsible

app = QApplication([])

collapsible = QCollapsible("Advanced analysis")
collapsible.addWidget(QLabel("This is the inside of the collapsible frame"))
for i in range(10):
    collapsible.addWidget(QPushButton(f"Content button {i + 1}"))

collapsible.expand(animate=False)
collapsible.show()
app.exec_()



from qtpy.QtCore import Signal
from qtpy.QtWidgets import QApplication, QWidget

from superqt.utils import qthrottled


class Demo(QWidget):
    positionChanged = Signal(int, int)

    def __init__(self) -> None:
        super().__init__()
        self.setMouseTracking(True)
        self.positionChanged.connect(self._show_location)

    @qthrottled(timeout=400)  # call this no more than once every 400ms
    def _show_location(self, x, y):
        print("Throttled event at", x, y)

    def mouseMoveEvent(self, event):
        print("real move event at", event.x(), event.y())
        self.positionChanged.emit(event.x(), event.y())


if __name__ == "__main__":
    app = QApplication([])
    w = Demo()
    w.resize(600, 600)
    w.show()
    app.exec_()



from PyQt5.QtGui import QColor, QPalette
from qtpy.QtWidgets import QApplication, QTextEdit

from superqt.utils import CodeSyntaxHighlight

app = QApplication([])

text_area = QTextEdit()

highlight = CodeSyntaxHighlight(text_area.document(), "python", "monokai")

palette = text_area.palette()
palette.setColor(QPalette.Base, QColor(highlight.background_color))
text_area.setPalette(palette)
text_area.setText(
    """from argparse import ArgumentParser

def main():
    parser = ArgumentParser()
    parser.add_argument("name", help="Your name")
    args = parser.parse_args()
    print(f"Hello {args.name}")


if __name__ == "__main__":
    main()
"""
)

text_area.show()

app.exec_()



from unittest.mock import Mock

from superqt.utils import qdebounced, qthrottled


def test_debounced(qtbot):
    mock1 = Mock()
    mock2 = Mock()

    @qdebounced(timeout=5)
    def f1() -> str:
        mock1()

    def f2() -> str:
        mock2()

    for _ in range(10):
        f1()
        f2()

    qtbot.wait(5)
    mock1.assert_called_once()
    assert mock2.call_count == 10


def test_throttled(qtbot):
    mock1 = Mock()
    mock2 = Mock()

    @qthrottled(timeout=5)
    def f1() -> str:
        mock1()

    def f2() -> str:
        mock2()

    for _ in range(10):
        f1()
        f2()

    qtbot.wait(5)
    assert mock1.call_count == 2
    assert mock2.call_count == 10



"""A test module for testing collapsible"""

from qtpy.QtCore import QEasingCurve
from qtpy.QtWidgets import QPushButton

from superqt import QCollapsible


def test_checked_initialization(qtbot):
    """Test simple collapsible"""
    wdg1 = QCollapsible("Advanced analysis")
    wdg1.expand(False)
    assert wdg1.isExpanded()
    assert wdg1._content.maximumHeight() > 0

    wdg2 = QCollapsible("Advanced analysis")
    wdg1.collapse(False)
    assert not wdg2.isExpanded()
    assert wdg2._content.maximumHeight() == 0


def test_content_hide_show(qtbot):
    """Test collapsible with content"""

    # Create child component
    collapsible = QCollapsible("Advanced analysis")
    for i in range(10):
        collapsible.addWidget(QPushButton(f"Content button {i + 1}"))

    collapsible.collapse(False)
    assert not collapsible.isExpanded()
    assert collapsible._content.maximumHeight() == 0

    collapsible.expand(False)
    assert collapsible.isExpanded()
    assert collapsible._content.maximumHeight() > 0


def test_locking(qtbot):
    """Test locking collapsible"""
    wdg1 = QCollapsible()
    assert wdg1.locked() is False
    wdg1.setLocked(True)
    assert wdg1.locked() is True
    assert not wdg1.isExpanded()

    wdg1._toggle_btn.setChecked(True)
    assert not wdg1.isExpanded()

    wdg1._toggle()
    assert not wdg1.isExpanded()

    wdg1.expand()
    assert not wdg1.isExpanded()

    wdg1._toggle_btn.setChecked(False)
    assert not wdg1.isExpanded()

    wdg1.setLocked(False)
    wdg1.expand()
    assert wdg1.isExpanded()
    assert wdg1._toggle_btn.isChecked()


def test_changing_animation_settings(qtbot):
    """Quick test for changing animation settings"""
    wdg = QCollapsible()
    wdg.setDuration(600)
    wdg.setEasingCurve(QEasingCurve.Type.InElastic)
    assert wdg._animation.easingCurve() == QEasingCurve.Type.InElastic
    assert wdg._animation.duration() == 600


def test_changing_content(qtbot):
    """Test changing the content"""
    content = QPushButton()
    wdg = QCollapsible()
    wdg.setContent(content)
    assert wdg._content == content


def test_changing_text(qtbot):
    """Test changing the content"""
    wdg = QCollapsible()
    wdg.setText("Hi new text")
    assert wdg.text() == "Hi new text"
    assert wdg._toggle_btn.text() == QCollapsible._COLLAPSED + "Hi new text"
